from .categories import Categories

class Emoji:
    def __init__(
        self, 
        data: dict
        ) -> None:
        self._raw = data
        self.id = data.pop('id')
        self.title = data.pop('title')
        self.slug = data.pop('slug')
        self.image = data.pop('image')
        self.description = data.pop('description')
        self.license = data.pop('license')
        self.source = data.pop('source')
        self.faves = data.pop('faves')
        self.submitted_by = data.pop('submitted_by')
        self.width = data.pop('width')
        self.height = data.pop('height')
        self.filesize = data.pop('filesize')
        
    @property
    def formatted_description(self) -> str:
        """
        Returns
        -------
        str
            The emojis description formatted correctly with caps.
        """
        return self.description.capitalize()

    @property
    def category(self) -> str:
        category = self._raw.get('category')
        return Categories().types.get(str(category))


import aiohttp
from attr import has
from .http import HTTP
from .pack import Pack
from .emoji import Emoji
from .errors import WrongType

from typing import (
    Optional, 
    List, 
    Union
)


class Client(HTTP):
    def __init__(
        self, 
        session: Optional[aiohttp.ClientSession] = None
    ) -> None:
        super().__init__(session=session or aiohttp.ClientSession())

    async def get_pack_from(
        self, 
        attr: str, 
        value: str,
        *,
        packs: List[Pack] = []
    ) -> Union[None, Emoji]:
        """
        Get pack from an attribute of the Pack.
        
        Parameters
        ----------
        attr: str
            The attribute of the Pack you want to search for
        value: str
            The vlaue you want the attribute to be.
        packs: List[Pack]
            The optional list of packs to search through. If none is specified a request will be made to get all packs.
            
        Returns
        -------
        Optional[Pack]
        """
        packs = await self.fetch_packs() if not packs else packs
        if not hasattr(packs[0], attr):
            raise WrongType(f"class Pack does not have attribute {attr}")
        
        for pack in packs:
            if getattr(pack, attr) == value:
                return pack
        
        return None

    


class Pack:
    def __init__(
        self,
        data: dict
    ) -> None:
        self._raw = data
        self.id = data.pop('id')
        self.name = data.pop('name')
        self.description = data.pop('description')
        self.slug = data.pop('slug')
        self.image = data.pop('image')
        self.emojis = data.pop('emojis')
        self.download = data.pop('download')
        self.amount = data.pop('amount')
        
    



class NotImplemented(Exception):
    def __init__(self, message, *args):
        self.message = message
        self.args = args
    
    def __str__(self) -> str:
        return self.message
    
class WrongType(Exception):
    pass


import inspect

class Categories:
    OriginalStyle = '1'
    TvMovie = '2'
    Meme = '3'
    Anime = '4'
    Celebrity = '5'
    Blobs = '6'
    Thinking = '7'
    Animated = '8'
    NSFW = '9'
    Gaming = '10'
    Letters = '11'
    Other = '12'
    Pepe = '13'
    Logos = '14'
    Cute = '15'
    Utility = '16'
    Animals = '17'
    Recolors = '18'
    Flags = '19'
    Hearts = '20'
        
    @property
    def types(self):
        """
        Get all category types. We do some magic for this.
        """
        mems = [mem for mem in inspect.getmembers(Categories, lambda a: not(inspect.isroutine(a))) if isinstance(mem[0], str) and isinstance(mem[1], str) and mem[1].isdigit()]
        return {num: key for key, num in mems}


import aiohttp
from typing import Optional, List, Dict

from .emoji import Emoji
from .categories import Categories
from .pack import Pack
from .errors import NotImplemented

BASE = 'https://emoji.gg/api'

class Route:
    def __init__(
        self, 
        ext: str = '', 
        **kwargs
    ) -> None:
        self.ext = ext
        self.kwargs = kwargs
    
    @property
    def url(self) -> str:
        return BASE + self.ext


class HTTP:
    def __init__(
        self, 
        session: Optional[aiohttp.ClientSession] = None
    ) -> None:
        self.session = session or aiohttp.ClientSession()
        
    async def request(
        self, 
        method: str, 
        route: Route
    ) -> Dict:
        async with self.session.request(method, route.url, **route.kwargs) as resp:
            if resp.status != 200:
                raise NotImplemented(
                    'This error has not been implemented yet. ' \
                    'We actually dont know the response of this... please give it to the lib deb', 
                    await resp.read()  # I don't know if this is json, we'll return the read version instead.
                )
            return await resp.json() 
    
    async def fetch_emojis(self) -> List[Emoji]:
        """
        Retreives approx 5000 emojis from the website.
        
        Returns
        -------
        List[Emoji]
        """
        data = await self.request('GET', Route('/'))
        return [Emoji(entry) for entry in data]
    
    async def fetch_packs(self):
        """
        Retreives packs from the website.
        
        Returns
        -------
        List[Pack]
        """
        data = await self.request('GET', Route('/packs'))
        return [Pack(entry) for entry in data]
    
    async def fetch_statistics(self):
        """
        Retreives statistics about this website.
        
        Returns
        -------
        dict
        """
        return await self.request('GET', Route(params={
            'request': 'stats'
        }))
        
    async def fetch_categories(self):
        """
        Retreives categories from the website. Fetches the current categories
        
        Returns
        -------
        dict
        """
        return await self.request('GET', Route(params={
            'request': 'categories'
        }))